package entities;


import java.io.Serializable;
import java.util.Date;
import java.util.Set;
import java.util.UUID;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.Table;

import org.hibernate.annotations.Cascade;
import org.hibernate.annotations.GenericGenerator;
import org.hibernate.annotations.Proxy;
import org.hibernate.annotations.Type;

import com.fasterxml.jackson.annotation.JsonIdentityInfo;
import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.ObjectIdGenerators;
import com.fasterxml.jackson.annotation.JsonInclude.Include;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SerializationFeature;


@Entity
@Table(name="\"DMP\"")
@JsonIdentityInfo(generator=ObjectIdGenerators.PropertyGenerator.class, property="id")
public class DMP implements Serializable {
	

	private static final long serialVersionUID = -8263056535208547615L;
	
	
	@Id
	@GeneratedValue
    @GenericGenerator(name = "uuid2", strategy = "uuid2")
    @Column(name = "\"ID\"", updatable = false, nullable = false, columnDefinition = "BINARY(16)")
	private UUID id;
	
	@Type(type="org.hibernate.type.PostgresUUIDType")
    @Column(name = "\"Previous\"")
	private UUID previous;
    
	@Column(name = "\"Label\"")
	private String label;
	
	@Column(name = "\"Version\"")
	private Integer  version;

	@OneToMany(mappedBy = "dmp", fetch = FetchType.LAZY)
	private Set<Dataset> dataset;
	
	
	@ManyToOne(fetch = FetchType.EAGER)
    @JoinColumn(name = "\"Project\"")
	private Project project;
	
	
	@Type(type="typedefinition.XMLType")
	@Column(name = "\"ProfileData\"", columnDefinition = "xml", nullable = true)
	private String profileData;

	@ManyToOne(fetch = FetchType.EAGER)
    @JoinColumn(name = "\"Profile\"")
	private DMPProfile profile;
	
	
	@OneToMany(fetch = FetchType.LAZY)
    @JoinTable(name="\"DMPOrganisation\"", 
    		joinColumns={@JoinColumn(name="\"DMP\"", referencedColumnName="\"ID\"")},
    	    inverseJoinColumns={@JoinColumn(name="\"Organisation\"", referencedColumnName="\"ID\"")}
    )  
    private Set<Organisation> organisations;
	
	
	@OneToMany(fetch = FetchType.LAZY)
    @JoinTable(name="\"DMPResearcher\"", 
    		joinColumns={@JoinColumn(name="\"DMP\"", referencedColumnName="\"ID\"")},
    	    inverseJoinColumns={@JoinColumn(name="\"Researcher\"", referencedColumnName="\"ID\"")}
    )  
    private Set<Researcher> researchers;
	
	
	@OneToMany(fetch = FetchType.LAZY)
    @JoinTable(name="\"UserDMP\"",
    		joinColumns={@JoinColumn(name="dmp", referencedColumnName="\"ID\"")},
    	    inverseJoinColumns={@JoinColumn(name="usr", referencedColumnName="id")}
    )
    private Set<UserInfo> users;
	
	
	@Column(name = "\"Status\"", nullable = false)
	private Short status; 
	
	
	@Column(name = "\"Created\"")
	private Date created = null;
	
	@Column(name = "\"Modified\"")
	private Date modified = new Date();
	
	
	@Column(name = "\"Description\"")
	private String description;
	
	
	public String getDescription() {
		return description;
	}


	public void setDescription(String description) {
		this.description = description;
	}
	
	

	public Short getStatus() {
		return status;
	}


	public void setStatus(Short status) {
		this.status = status;
	}


	public Date getCreated() {
		return created;
	}


	public void setCreated(Date created) {
		this.created = created;
	}


	public Date getModified() {
		return modified;
	}


	public void setModified(Date modified) {
		this.modified = modified;
	}
	
	
	
	public Set<UserInfo> getUsers() {
		return users;
	}
	
	public void setUsers(Set<UserInfo> users) {
		this.users = users;
	}
	
	
	public UUID getId() {
		return id;
	}

	public void setId(UUID id) {
		this.id = id;
	}

	public UUID getPrevious() {
		return previous;
	}

	public void setPrevious(UUID previous) {
		this.previous = previous;
	}

	public String getLabel() {
		return label;
	}

	public void setLabel(String label) {
		this.label = label;
	}

	public Integer getVersion() {
		return version;
	}

	public void setVersion(Integer version) {
		this.version = version;
	}

	public Project getProject() {
		return project;
	}

	public void setProject(Project project) {
		this.project = project;
	}

	public String getProfileData() {
		return profileData;
	}

	public void setProfileData(String profileData) {
		this.profileData = profileData;
	}

	public DMPProfile getProfile() {
		return profile;
	}

	public void setProfile(DMPProfile profile) {
		this.profile = profile;
	}

	public Set<Dataset> getDataset() {
		return dataset;
	}

	public void setDataset(Set<Dataset> dataset) {
		this.dataset = dataset;
	}
	
	public Set<Organisation> getOrganisations() {
		return organisations;
	}

	public void setOrganisations(Set<Organisation> organisations) {
		this.organisations = organisations;
	}

	public Set<Researcher> getResearchers() {
		return researchers;
	}

	public void setResearchers(Set<Researcher> researchers) {
		this.researchers = researchers;
	}
	


/*
	public String toString() {
		try {
			return new ObjectMapper().enable(SerializationFeature.INDENT_OUTPUT).writeValueAsString(this);
		} catch (JsonProcessingException e) {
			return "";
		}
	}
	
	*/
	
}
