package security.validators;

import java.io.IOException;
import java.security.GeneralSecurityException;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;

import com.google.api.client.googleapis.auth.oauth2.GoogleIdToken;
import com.google.api.client.googleapis.auth.oauth2.GoogleIdTokenVerifier;
import com.google.api.client.googleapis.auth.oauth2.GoogleIdToken.Payload;
import com.google.api.client.http.HttpTransport;
import com.google.api.client.http.javanet.NetHttpTransport;
import com.google.api.client.json.jackson2.JacksonFactory;

import dao.entities.UserInfoDao;
import entities.UserInfo;
import exceptions.NonValidTokenException;

public class GoogleTokenValidator implements TokenValidator {

	private static final JacksonFactory jacksonFactory = new JacksonFactory();
	private static final HttpTransport transport = new NetHttpTransport();

	@Autowired private UserInfoDao userInfoDao;
	
	
	private static final List<String> clientIDs = Arrays.asList(
			"1010962018903-glegmqudqtl1lub0150vacopbu06lgsg.apps.googleusercontent.com",
			"1010962018903-glegmqudqtl1lub0150vacopbu06lgsg.apps.googleusercontent.com"
			);
	
	private GoogleIdTokenVerifier verifier = null;
	
	
	public GoogleTokenValidator() {
		verifier = new GoogleIdTokenVerifier.Builder(transport, jacksonFactory)
			    .setAudience(clientIDs)
			    // Or, if multiple clients access the backend:
			    //.setAudience(Arrays.asList(CLIENT_ID_1, CLIENT_ID_2, CLIENT_ID_3))
			    .build();
	}
	
	
	@Override
	public UserInfo validateToken(String token) throws NonValidTokenException {
		
		GoogleIdToken idToken = null;
		try {
			idToken = verifier.verify(token);
		}
		catch(GeneralSecurityException ex) {
			throw new NonValidTokenException("Token is not valid -> "+ex.getMessage());
		}
		catch(IOException ex) {
			throw new NonValidTokenException("Could not verify token -> "+ex.getMessage());
		}
		catch(IllegalArgumentException ex) {
			throw new NonValidTokenException("Could not verify token");
		}
		
		if(idToken == null) {
			throw new NonValidTokenException("Not a valid token");
		}

		Payload payload = idToken.getPayload();

		UserInfo userInfo = userInfoDao.getByMail(payload.getEmail());

		if(userInfo == null) { //means not existing in db, so create one
			userInfo = new UserInfo();
			userInfo.setName((String)payload.get("name"));
			userInfo.setVerified_email(payload.getEmailVerified());
			userInfo.setEmail(payload.getEmail());
			userInfo.setCreated(new Date());
			userInfo.setLastloggedin(new Date());
			userInfo.setAuthorization_level(new Short("1"));
			userInfo.setUsertype(new Short("1"));
			userInfo = userInfoDao.create(userInfo);
		}
		else {
			userInfo.setLastloggedin(new Date());
			userInfo = userInfoDao.update(userInfo);
		}
		
		return userInfo;
	  
	}
	
	
	
	
	
}
